@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,8 +14,9 @@
 
 import subprocess
 import os
+from typing import NamedTuple, Optional
 
-version_info = (0, 6, 11)
+version_info = (0, 7, 1)
 _num_index = max(idx if isinstance(v, int) else 0
                  for idx, v in enumerate(version_info))
 __version__ = '.'.join(map(str, version_info[:_num_index + 1])) + \
@@ -31,7 +32,12 @@ def _get_cmd_results(pkg_root, cmd):
         return s
 
 
-def get_git_info():
+class GitInfo(NamedTuple):
+    commit_hash: str
+    commit_ref: str
+
+
+def get_git_info() -> Optional[GitInfo]:
     pkg_root = os.path.dirname(os.path.abspath(__file__))
     git_root = os.path.join(os.path.dirname(pkg_root), '.git')
 
@@ -52,11 +58,11 @@ def get_git_info():
                 commit_ref = commit_ref.rstrip(')')
         if commit_ref is None:
             return
-        return commit_hash, commit_ref
+        return GitInfo(commit_hash, commit_ref)
     else:
         branch_file = os.path.join(pkg_root, '.git-branch')
         if not os.path.exists(branch_file):
             return
         with open(branch_file, 'r') as bf:
             bf_content = bf.read().strip()
-            return bf_content.split(None, 1)
+            return GitInfo(*bf_content.split(None, 1))
