@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,14 +24,15 @@ from copy import deepcopy
 from typing import Union, Dict
 
 
-_DEFAULT_REDIRECT_WARN = 'Option {source} has been replaced by {target} and might be removed in a future release.'
+_DEFAULT_REDIRECT_WARN = 'Option {source} has been replaced by {target} and ' \
+                         'might be removed in a future release.'
 
 
 class OptionError(Exception):
     pass
 
 
-class Redirection(object):
+class Redirection:
     def __init__(self, item, warn=None):
         self._items = item.split('.')
         self._warn = warn
@@ -158,7 +159,7 @@ class AttributeDict(dict):
         return d
 
 
-class Config(object):
+class Config:
     def __init__(self, config=None):
         self._config = config or AttributeDict()
         self._serialize_options = []
@@ -167,7 +168,8 @@ class Config(object):
         return list(self._config.keys())
 
     def __getattr__(self, item):
-        return getattr(self._config, item)
+        config = object.__getattribute__(self, '_config')
+        return getattr(config, item)
 
     def __setattr__(self, key, value):
         if key.startswith('_'):
@@ -259,7 +261,7 @@ class Config(object):
 
 @contextlib.contextmanager
 def option_context(config=None):
-    global_options = _get_global_option()
+    global_options = get_global_option()
 
     try:
         config = config or dict()
@@ -314,7 +316,8 @@ default_options.register_option('tcp_timeout', 30, validator=is_integer)
 default_options.register_option('verbose', False, validator=is_bool)
 default_options.register_option('kv_store', ':inproc:', validator=is_string)
 default_options.register_option('check_interval', 20, validator=is_integer)
-default_options.register_option('serialize_method', 'json')
+default_options.register_option('show_progress', 'auto', validator=any_validator(is_bool, is_string))
+default_options.register_option('serialize_method', 'pickle')
 
 # dataframe-related options
 default_options.register_option('dataframe.mode.use_inf_as_na', False, validator=is_bool)
@@ -422,7 +425,7 @@ _options_local = threading.local()
 _options_local.default_options = default_options
 
 
-def _get_global_option():
+def get_global_option():
     ret = getattr(_options_local, 'default_options', None)
     if ret is None:
         ret = _options_local.default_options = Config(deepcopy(default_options._config))
@@ -430,15 +433,15 @@ def _get_global_option():
     return ret
 
 
-class OptionsProxy(object):
+class OptionsProxy:
     def __dir__(self):
-        return dir(_get_global_option())
+        return dir(get_global_option())
 
     def __getattribute__(self, attr):
-        return getattr(_get_global_option(), attr)
+        return getattr(get_global_option(), attr)
 
     def __setattr__(self, key, value):
-        setattr(_get_global_option(), key, value)
+        setattr(get_global_option(), key, value)
 
 
 options = OptionsProxy()
