@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,11 +14,11 @@
 
 from ... import opcodes as OperandDef
 from ...utils import classproperty
-from .core import DataFrameBinOpMixin, DataFrameBinOp
+from .core import DataFrameBinopUfunc
 from .docstring import bin_compare_doc
 
 
-class DataFrameNotEqual(DataFrameBinOp, DataFrameBinOpMixin):
+class DataFrameNotEqual(DataFrameBinopUfunc):
     _op_type_ = OperandDef.NE
 
     _func_name = 'ne'
@@ -28,6 +28,11 @@ class DataFrameNotEqual(DataFrameBinOp, DataFrameBinOpMixin):
     def _operator(self):
         return lambda lhs, rhs: lhs.ne(rhs)
 
+    @classproperty
+    def tensor_op_type(self):
+        from ...tensor.arithmetic import TensorNotEqual
+        return TensorNotEqual
+
 
 _ne_example = """
 >>> a.ne(b, fill_value=0).execute()
