@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,6 @@ import itertools
 import operator
 import re
 from copy import copy as copy_obj
-from distutils.version import LooseVersion
 from numbers import Integral
 from typing import Type, Sequence
 
@@ -44,9 +43,10 @@ except ImportError:  # pragma: no cover
     pa_null = None
 
 from ..config import options
-from ..utils import is_kernel_mode
+from ..core import is_kernel_mode
+from ..lib.version import parse as parse_version
 
-_use_bool_any_all = LooseVersion(pd.__version__) >= '1.3.0'
+_use_bool_any_all = parse_version(pd.__version__) >= parse_version('1.3.0')
 
 
 class ArrowDtype(ExtensionDtype):
@@ -614,7 +614,7 @@ class ArrowStringArray(ArrowArray, StringArrayBase):
         else:
             self._ndarray[key] = value
 
-    # Overrride parent because we have different return types.
+    # Override parent because we have different return types.
     @classmethod
     def _create_arithmetic_method(cls, op):
         # Note: this handles both arithmetic and comparison methods.
