@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,11 +16,10 @@ import numpy as np
 import pandas as pd
 
 from ... import opcodes
-from ...custom_log import redirect_custom_log
-from ...serialize import KeyField, FunctionField, TupleField, \
-    DictField, StringField
-from ...tiles import TilesError
-from ...utils import check_chunks_unknown_shape, enter_current_session, quiet_stdio
+from ...core import recursive_tile
+from ...core.custom_log import redirect_custom_log
+from ...serialization.serializables import KeyField, FunctionField, TupleField, DictField
+from ...utils import has_unknown_shape, enter_current_session, quiet_stdio
 from ..operands import DataFrameOperand, DataFrameOperandMixin, OutputType
 from ..utils import build_df, build_series, build_empty_df, parse_index, \
     validate_output_types
@@ -34,15 +33,13 @@ class DataFrameCartesianChunk(DataFrameOperand, DataFrameOperandMixin):
     _func = FunctionField('func')
     _args = TupleField('args')
     _kwargs = DictField('kwargs')
-    # for chunk
-    _tileable_op_key = StringField('tileable_op_key')
 
     def __init__(self, left=None, right=None, func=None, args=None, kwargs=None,
-                 memory_scale=None, output_types=None, tileable_op_key=None, **kw):
+                 output_types=None, **kw):
         super().__init__(_left=left, _right=right, _func=func, _args=args,
-                         _kwargs=kwargs, _memory_scale=memory_scale,
-                         _output_types=output_types,
-                         _tileable_op_key=tileable_op_key, **kw)
+                         _kwargs=kwargs, _output_types=output_types, **kw)
+        if self.memory_scale is None:
+            self.memory_scale = 2.0
 
     @property
     def left(self):
@@ -64,14 +61,6 @@ class DataFrameCartesianChunk(DataFrameOperand, DataFrameOperandMixin):
     def kwargs(self):
         return self._kwargs
 
-    @property
-    def memory_scale(self):
-        return self._memory_scale or 2.0
-
-    @property
-    def tileable_op_key(self):
-        return self._tileable_op_key
-
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
         self._left = self._inputs[0]
@@ -131,13 +120,17 @@ class DataFrameCartesianChunk(DataFrameOperand, DataFrameOperandMixin):
         out = op.outputs[0]
 
         if left.ndim == 2 and left.chunk_shape[1] > 1:
-            check_chunks_unknown_shape([left], TilesError)
+            if has_unknown_shape(left):
+                yield
             # if left is a DataFrame, make sure 1 chunk on axis columns
-            left = left.rechunk({1: left.shape[1]})._inplace_tile()
+            left = yield from recursive_tile(
+                left.rechunk({1: left.shape[1]}))
         if right.ndim == 2 and right.chunk_shape[1] > 1:
-            check_chunks_unknown_shape([right], TilesError)
+            if has_unknown_shape(right):
+                yield
             # if right is a DataFrame, make sure 1 chunk on axis columns
-            right = right.rechunk({1: right.shape[1]})._inplace_tile()
+            right = yield from recursive_tile(
+                right.rechunk({1: right.shape[1]}))
 
         out_chunks = []
         nsplits = [[]] if out.ndim == 1 else [[], [out.shape[1]]]
@@ -145,7 +138,7 @@ class DataFrameCartesianChunk(DataFrameOperand, DataFrameOperandMixin):
         for left_chunk in left.chunks:
             for right_chunk in right.chunks:
                 chunk_op = op.copy().reset_key()
-                chunk_op._tileable_op_key = op.key
+                chunk_op.tileable_op_key = op.key
                 if op.output_types[0] == OutputType.dataframe:
                     shape = (np.nan, out.shape[1])
                     index_value = parse_index(out.index_value.to_pandas(),
