@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 import numpy as np
 
 from ... import opcodes
-from ...operands import OperandStage, OutputType
-from ...utils import get_shuffle_input_keys_idxes
+from ...core import OutputType
+from ...core.operand import OperandStage
 from ..utils import gen_unknown_index_value, hash_dataframe_on
 from ._duplicate import DuplicateOperand, validate_subset
 
@@ -25,24 +25,21 @@ class DataFrameDuplicated(DuplicateOperand):
     _op_type_ = opcodes.DUPLICATED
 
     def __init__(self, subset=None, keep=None, output_types=None,
-                 method=None, subset_chunk=None, shuffle_size=None,
-                 shuffle_phase=None, shuffle_key=None, **kw):
+                 method=None, subset_chunk=None, shuffle_size=None, **kw):
         super().__init__(_subset=subset, _keep=keep,
                          _output_types=output_types, _method=method,
                          _subset_chunk=subset_chunk,
-                         _shuffle_size=shuffle_size,
-                         _shuffle_phase=shuffle_phase,
-                         _shuffle_key=shuffle_key, **kw)
+                         _shuffle_size=shuffle_size, **kw)
 
     @classmethod
-    def _get_shape(cls, input_shape):
+    def _get_shape(cls, input_shape, op):
         return (input_shape[0],)
 
     @classmethod
     def _gen_tileable_params(cls, op: "DataFrameDuplicated", input_params):
         # duplicated() always returns a Series
         return {
-            'shape': cls._get_shape(input_params['shape']),
+            'shape': cls._get_shape(input_params['shape'], op),
             'index_value': input_params['index_value'],
             'dtype': np.dtype(bool),
             'name': input_params.get('name')
@@ -54,10 +51,17 @@ class DataFrameDuplicated(DuplicateOperand):
 
         return self.new_tileable([inp], kws=[params])
 
+    @classmethod
+    def _get_map_output_types(cls, input_chunk, method: str):
+        if method in ('tree', 'subset_tree'):
+            return [OutputType.dataframe]
+        else:
+            return input_chunk.op.output_types
+
     @classmethod
     def _gen_chunk_params_default(cls, op: "DataFrameDuplicated", input_chunk):
         return {
-            'shape': cls._get_shape(input_chunk.shape),
+            'shape': cls._get_shape(input_chunk.shape, op),
             'index_value': input_chunk.index_value,
             'dtype': np.dtype(bool),
             'name': input_chunk.name if input_chunk.ndim == 1 else None,
@@ -66,22 +70,23 @@ class DataFrameDuplicated(DuplicateOperand):
 
     @classmethod
     def _get_intermediate_shape(cls, input_shape):
-        return (np.nan,) + input_shape[1:]
+        if len(input_shape) > 1:
+            s = input_shape[1:]
+        else:
+            s = (2,)
+        return (np.nan,) + s
 
     @classmethod
     def _gen_intermediate_chunk_params(cls, op: "DataFrameDuplicated", input_chunk):
         inp = op.input
         chunk_params = dict()
         chunk_params['index'] = input_chunk.index[:1] + (0,) * (inp.ndim - 1)
-        chunk_params['shape'] = cls._get_intermediate_shape(input_chunk.shape)
+        chunk_params['shape'] = shape = cls._get_intermediate_shape(input_chunk.shape)
         chunk_params['index_value'] = gen_unknown_index_value(
             input_chunk.index_value, input_chunk)
-        if inp.ndim == 2:
+        if inp.ndim == 2 and len(shape) == 2:
             chunk_params['columns_value'] = input_chunk.columns_value
             chunk_params['dtypes'] = input_chunk.dtypes
-        else:
-            chunk_params['name'] = inp.name
-            chunk_params['dtype'] = inp.dtype
         return chunk_params
 
     @classmethod
@@ -94,7 +99,7 @@ class DataFrameDuplicated(DuplicateOperand):
             is_terminal_chunk = True
         elif op.method == 'tree' and op.stage == OperandStage.agg:
             is_terminal_chunk = True
-        elif op.method == 'shuffle' and op.shuffle_phase == 'put_back':
+        elif op.method == 'shuffle' and op.reducer_phase == 'put_back':
             is_terminal_chunk = True
 
         if is_terminal_chunk:
@@ -151,8 +156,9 @@ class DataFrameDuplicated(DuplicateOperand):
         duplicates = inp[~inp.iloc[:, -1]]
         dup_on_duplicated = cls._duplicated(duplicates, op)
         result[~inp.iloc[:, -1]] = dup_on_duplicated
-        if result.name == '_duplicated_':
-            result.name = None
+        expect_name = op.outputs[0].name
+        if result.name != expect_name:
+            result.name = expect_name
         result = result.astype(bool)
         ctx[op.outputs[0].key] = result
 
@@ -195,14 +201,14 @@ class DataFrameDuplicated(DuplicateOperand):
         result['_i_'] = np.arange(result.shape[0])
         hashed = hash_dataframe_on(result, subset, shuffle_size)
         for i, data in enumerate(hashed):
-            ctx[(out.key, str(i))] = result.iloc[data]
+            reducer_idx = (i,) + out.index[1:]
+            ctx[out.key, reducer_idx] = result.iloc[data]
 
     @classmethod
-    def _execute_shuffle_reduce(cls, ctx, op):
+    def _execute_shuffle_reduce(cls, ctx, op: "DataFrameDuplicated"):
         out = op.outputs[0]
+        inputs = list(op.iter_mapper_data(ctx))
 
-        input_keys, _ = get_shuffle_input_keys_idxes(op.inputs[0])
-        inputs = [ctx[(inp_key, str(out.index[0]))] for inp_key in input_keys]
         xdf = cls._get_xdf(inputs[0])
         inp = xdf.concat(inputs)
         subset = [c for c in inp.columns
@@ -216,13 +222,12 @@ class DataFrameDuplicated(DuplicateOperand):
                 filtered.columns = ['_duplicated_'] + filtered.columns[1:].tolist()
             else:
                 filtered.columns = [subset[0]] + filtered.columns[1:].tolist()
-            ctx[(out.key, str(i))] = filtered
+            ctx[out.key, (i,)] = filtered
 
     @classmethod
-    def _execute_shuffle_put_back(cls, ctx, op):
-        out = op.outputs[0]
-        input_keys, _ = get_shuffle_input_keys_idxes(op.inputs[0])
-        inputs = [ctx[(inp_key, str(out.index[0]))] for inp_key in input_keys]
+    def _execute_shuffle_put_back(cls, ctx, op: "DataFrameDuplicated"):
+        inputs = list(op.iter_mapper_data(ctx))
+
         xdf = cls._get_xdf(inputs[0])
         inp = xdf.concat(inputs)
         inp.sort_values('_i_', inplace=True)
@@ -233,7 +238,7 @@ class DataFrameDuplicated(DuplicateOperand):
         ctx[op.outputs[0].key] = duplicated
 
     @classmethod
-    def execute(cls, ctx, op):
+    def execute(cls, ctx, op: "DataFrameDuplicated"):
         if op.method is None:
             # one chunk
             cls._execute_chunk(ctx, op)
@@ -261,10 +266,10 @@ class DataFrameDuplicated(DuplicateOperand):
             assert op.method == 'shuffle'
             if op.stage == OperandStage.map:
                 cls._execute_shuffle_map(ctx, op)
-            elif op.shuffle_phase == 'drop_duplicates':
+            elif op.reducer_phase == 'drop_duplicates':
                 cls._execute_shuffle_reduce(ctx, op)
             else:
-                assert op.shuffle_phase == 'put_back'
+                assert op.reducer_phase == 'put_back'
                 cls._execute_shuffle_put_back(ctx, op)
 
 
