@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,17 +21,16 @@ import numpy as np
 import pandas as pd
 
 from ... import opcodes
-from ...core import Base, Entity, OutputType, get_output_types
-from ...operands import OperandStage
-from ...serialize import BoolField, DictField, Float32Field, KeyField, \
+from ...core import ENTITY_TYPE, OutputType, get_output_types, recursive_tile
+from ...core.operand import OperandStage, MapReduceOperand
+from ...serialization.serializables import BoolField, DictField, Float32Field, KeyField, \
     Int32Field, Int64Field, NDArrayField, StringField
 from ...tensor.operands import TensorShuffleProxy
 from ...tensor.random import RandomStateField
 from ...tensor.utils import gen_random_seeds
-from ...tiles import TilesError
-from ...utils import check_chunks_unknown_shape, get_shuffle_input_keys_idxes
+from ...utils import has_unknown_shape
 from ..initializer import Series as asseries
-from ..operands import DataFrameOperandMixin, DataFrameOperand, DataFrameMapReduceOperand
+from ..operands import DataFrameOperandMixin, DataFrameOperand
 from ..utils import parse_index
 
 _ILOC_COL_HEADER = '_gsamp_iloc_col_'
@@ -83,9 +82,9 @@ class GroupBySampleILoc(DataFrameOperand, DataFrameOperandMixin):
 
     def __init__(self, groupby_params=None, size=None, frac=None, replace=None,
                  weights=None, random_state=None, seed=None, errors=None,
-                 left_iloc_bound=None, random_col_id=None, stage=None, **kw):
+                 left_iloc_bound=None, random_col_id=None, **kw):
         super().__init__(_groupby_params=groupby_params, _size=size, _frac=frac,
-                         _seed=seed, _replace=replace, _weights=weights, _stage=stage,
+                         _seed=seed, _replace=replace, _weights=weights,
                          _random_state=random_state, _errors=errors,
                          _left_iloc_bound=left_iloc_bound,
                          _random_col_id=random_col_id, **kw)
@@ -138,7 +137,7 @@ class GroupBySampleILoc(DataFrameOperand, DataFrameOperandMixin):
         super()._set_inputs(inputs)
         input_iter = iter(inputs)
         next(input_iter)
-        if isinstance(self.weights, (Base, Entity)):
+        if isinstance(self.weights, ENTITY_TYPE):
             self._weights = next(input_iter)
 
     def __call__(self, df):
@@ -156,7 +155,8 @@ class GroupBySampleILoc(DataFrameOperand, DataFrameOperandMixin):
         iloc_col_header = _ILOC_COL_HEADER + str(op.random_col_id)
         weight_col_header = _WEIGHT_COL_HEADER + str(op.random_col_id)
 
-        check_chunks_unknown_shape([in_df], TilesError)
+        if has_unknown_shape(in_df):
+            yield
 
         if op.weights is None:
             weights_iter = itertools.repeat(None)
@@ -183,7 +183,7 @@ class GroupBySampleILoc(DataFrameOperand, DataFrameOperandMixin):
         for inp_chunk, weight_chunk in zip(in_df.chunks, weights_iter):
             new_op = op.copy().reset_key()
             new_op._left_iloc_bound = int(left_ilocs[inp_chunk.index[0]])
-            new_op._stage = OperandStage.map
+            new_op.stage = OperandStage.map
             new_op._output_types = [OutputType.dataframe]
 
             inp_chunks = [inp_chunk]
@@ -210,13 +210,14 @@ class GroupBySampleILoc(DataFrameOperand, DataFrameOperandMixin):
 
         groupby_params = op.groupby_params.copy()
         groupby_params.pop('selection', None)
-        grouped = map_df.groupby(**groupby_params)._inplace_tile()
+        grouped = yield from recursive_tile(
+            map_df.groupby(**groupby_params))
 
         result_chunks = []
         seeds = gen_random_seeds(len(grouped.chunks), op.random_state)
         for group_chunk, seed in zip(grouped.chunks, seeds):
             new_op = op.copy().reset_key()
-            new_op._stage = OperandStage.reduce
+            new_op.stage = OperandStage.reduce
             new_op._weights = None
             new_op._random_state = None
             new_op._seed = seed
@@ -269,7 +270,7 @@ class GroupBySampleILoc(DataFrameOperand, DataFrameOperandMixin):
                 ])
 
 
-class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
+class GroupBySample(MapReduceOperand, DataFrameOperandMixin):
     _op_code_ = opcodes.RAND_SAMPLE
     _op_module_ = 'dataframe.groupby'
 
@@ -287,13 +288,12 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
     _input_nsplits = NDArrayField('input_nsplits')
 
     def __init__(self, groupby_params=None, size=None, frac=None, replace=None,
-                 weights=None, random_state=None, seed=None, stage=None,
-                 errors=None, input_nsplits=None, shuffle_key=None, **kw):
+                 weights=None, random_state=None, seed=None,
+                 errors=None, input_nsplits=None, **kw):
         super().__init__(_groupby_params=groupby_params, _size=size, _frac=frac,
                          _seed=seed, _replace=replace, _weights=weights,
-                         _random_state=random_state, _stage=stage,
-                         _errors=errors, _input_nsplits=input_nsplits,
-                         _shuffle_key=shuffle_key, **kw)
+                         _random_state=random_state, _errors=errors,
+                         _input_nsplits=input_nsplits, **kw)
 
     @property
     def groupby_params(self):
@@ -335,7 +335,7 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
         super()._set_inputs(inputs)
         input_iter = iter(inputs)
         next(input_iter)
-        if isinstance(self.weights, (Base, Entity)):
+        if isinstance(self.weights, ENTITY_TYPE):
             self._weights = next(input_iter)
 
     def __call__(self, groupby):
@@ -356,7 +356,7 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
         params['index_value'] = parse_index(result_df.index_value.to_pandas()[:0])
 
         input_dfs = [df]
-        if isinstance(self.weights, (Base, Entity)):
+        if isinstance(self.weights, ENTITY_TYPE):
             input_dfs.append(self.weights)
 
         self._output_types = get_output_types(result_df)
@@ -367,12 +367,12 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
         out = op.outputs[0]
 
         input_dfs = [in_df]
-        if isinstance(weights, (Base, Entity)):
+        if isinstance(weights, ENTITY_TYPE):
             input_dfs.append(weights)
 
         params = out.params
         chunk_op = op.copy().reset_key()
-        if isinstance(weights, (Base, Entity)):
+        if isinstance(weights, ENTITY_TYPE):
             chunk_op._weights = weights
         params['index'] = (0,) * out.ndim
         chunk = chunk_op.new_chunk([c.chunks[0] for c in input_dfs], **params)
@@ -384,19 +384,20 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
     @classmethod
     def _tile_distributed(cls, op: "GroupBySample", in_df, weights):
         out_df = op.outputs[0]
-        check_chunks_unknown_shape([in_df], TilesError)
+        if has_unknown_shape(in_df):
+            yield
 
         sample_iloc_op = GroupBySampleILoc(
             groupby_params=op.groupby_params, size=op.size, frac=op.frac, replace=op.replace,
             weights=weights, random_state=op.random_state, errors=op.errors, seed=None,
             left_iloc_bound=None
         )
-        sampled_iloc = sample_iloc_op(in_df)._inplace_tile()
+        sampled_iloc = yield from recursive_tile(sample_iloc_op(in_df))
 
         map_chunks = []
         for c in sampled_iloc.chunks:
             new_op = op.copy().reset_key()
-            new_op._stage = OperandStage.map
+            new_op.stage = OperandStage.map
             new_op._weights = None
             new_op._output_types = [OutputType.tensor]
             new_op._input_nsplits = np.array(in_df.nsplits[0])
@@ -410,9 +411,10 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
         reduce_chunks = []
         for src_chunk in in_df.chunks:
             new_op = op.copy().reset_key()
-            new_op._stage = OperandStage.reduce
             new_op._weights = None
-            new_op._shuffle_key = str(src_chunk.index[0])
+            new_op._output_types = [OutputType.tensor]
+            new_op.stage = OperandStage.reduce
+            new_op.reducer_index = (src_chunk.index[0],)
             new_op._input_nsplits = np.array(in_df.nsplits[0])
 
             reduce_chunks.append(new_op.new_chunk(
@@ -421,10 +423,10 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
         combine_chunks = []
         for src_chunk, reduce_chunk in zip(in_df.chunks, reduce_chunks):
             new_op = op.copy().reset_key()
-            new_op._stage = OperandStage.combine
+            new_op.stage = OperandStage.combine
             new_op._weights = None
 
-            params = src_chunk.params
+            params = out_df.params
             if out_df.ndim == 2:
                 params.update(dict(
                     index=src_chunk.index, dtypes=out_df.dtypes,
@@ -450,15 +452,17 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
     def tile(cls, op: 'GroupBySample'):
         in_df = op.inputs[0]
         if in_df.ndim == 2:
-            in_df = in_df.rechunk({1: (in_df.shape[1],)})._inplace_tile()
+            in_df = yield from recursive_tile(
+                in_df.rechunk({1: (in_df.shape[1],)}))
 
         weights = op.weights
-        if isinstance(weights, (Base, Entity)):
-            weights = weights.rechunk({0: in_df.nsplits[0]})._inplace_tile()
+        if isinstance(weights, ENTITY_TYPE):
+            weights = yield from recursive_tile(
+                weights.rechunk({0: in_df.nsplits[0]}))
 
         if len(in_df.chunks) == 1:
             return cls._tile_one_chunk(op, in_df, weights)
-        return cls._tile_distributed(op, in_df, weights)
+        return (yield from cls._tile_distributed(op, in_df, weights))
 
     @classmethod
     def execute(cls, ctx, op: 'GroupBySample'):
@@ -471,12 +475,9 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
             pos_array = np.cumsum([0] + input_nsplits)
             poses = np.searchsorted(in_data, pos_array).tolist()
             for idx, (left, right) in enumerate(zip(poses, poses[1:])):
-                ctx[(op.outputs[0].key, str(idx))] = in_data[left:right]
+                ctx[op.outputs[0].key, (idx,)] = in_data[left:right]
         elif op.stage == OperandStage.reduce:
-            in_indexes = []
-            input_keys, _ = get_shuffle_input_keys_idxes(op.inputs[0])
-            for input_key in input_keys:
-                in_indexes.append(ctx[(input_key, op.shuffle_key)])
+            in_indexes = list(op.iter_mapper_data(ctx))
             idx = np.sort(np.concatenate(in_indexes))
             if op.outputs[0].index[0] > 0:
                 acc_nsplits = np.cumsum(op.input_nsplits)
@@ -492,7 +493,7 @@ class GroupBySample(DataFrameMapReduceOperand, DataFrameOperandMixin):
         else:
             in_data = ctx[op.inputs[0].key]
             weights = op.weights
-            if isinstance(weights, (Base, Entity)):
+            if isinstance(weights, ENTITY_TYPE):
                 weights = ctx[weights.key]
             params = op.groupby_params.copy()
             selection = params.pop('selection', None)
@@ -601,7 +602,7 @@ def groupby_sample(groupby, n=None, frac=None, replace=False, weights=None,
     groupby_params = groupby.op.groupby_params.copy()
     groupby_params.pop('as_index', None)
 
-    if weights is not None and not isinstance(weights, (Base, Entity)):
+    if weights is not None and not isinstance(weights, ENTITY_TYPE):
         weights = asseries(weights)
 
     n = 1 if n is None and frac is None else n
