@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,10 +15,12 @@
 import numpy as np
 import pandas as pd
 
-from ..core import Base, Entity, OutputType
+from ..core import ENTITY_TYPE
+from ..serialization.serializables import SerializableMeta
 from ..tensor import tensor as astensor, stack
+from ..tensor.array_utils import is_cupy
 from ..tensor.core import TENSOR_TYPE
-from ..utils import ceildiv
+from ..utils import ceildiv, lazy_import
 from .core import DATAFRAME_TYPE, SERIES_TYPE, INDEX_TYPE, DataFrame as _Frame, \
     Series as _Series, Index as _Index
 from .datasource.dataframe import from_pandas as from_pandas_df
@@ -27,10 +29,12 @@ from .datasource.index import from_pandas as from_pandas_index, \
     from_tileable as from_tileable_index
 from .datasource.from_tensor import dataframe_from_tensor, series_from_tensor, \
     dataframe_from_1d_tileables
-from .fetch import DataFrameFetch
+from .utils import is_index, is_cudf
+
+cudf = lazy_import('cudf', globals=globals())
 
 
-class InitializerMeta(type):
+class InitializerMeta(SerializableMeta):
     def __instancecheck__(cls, instance):
         return isinstance(instance, (cls.__base__,) + getattr(cls, '_allow_data_type_'))
 
@@ -38,9 +42,6 @@ class InitializerMeta(type):
 class DataFrame(_Frame, metaclass=InitializerMeta):
     def __init__(self, data=None, index=None, columns=None, dtype=None, copy=False,
                  chunk_size=None, gpu=None, sparse=None, num_partitions=None):
-        # make sure __getattr__ does not result in stack overflow
-        self._data = None
-
         need_repart = False
         if isinstance(data, TENSOR_TYPE):
             if chunk_size is not None:
@@ -57,12 +58,12 @@ class DataFrame(_Frame, metaclass=InitializerMeta):
             else:
                 df = data
             need_repart = num_partitions is not None
-        elif isinstance(data, dict) and any(isinstance(v, (Base, Entity)) for v in data.values()):
+        elif isinstance(data, dict) and any(isinstance(v, ENTITY_TYPE) for v in data.values()):
             # data is a dict and some value is tensor
             df = dataframe_from_1d_tileables(
                 data, index=index, columns=columns, gpu=gpu, sparse=sparse)
             need_repart = num_partitions is not None
-        elif isinstance(data, list) and any(isinstance(v, (Base, Entity)) for v in data):
+        elif isinstance(data, list) and any(isinstance(v, ENTITY_TYPE) for v in data):
             # stack data together
             data = stack(data)
             df = dataframe_from_tensor(data, index=index, columns=columns,
@@ -80,7 +81,12 @@ class DataFrame(_Frame, metaclass=InitializerMeta):
                                            columns=columns, gpu=gpu, sparse=sparse)
             need_repart = num_partitions is not None
         else:
-            pdf = pd.DataFrame(data, index=index, columns=columns, dtype=dtype, copy=copy)
+            if is_cudf(data) or is_cupy(data):  # pragma: no cover
+                pdf = cudf.DataFrame(data, index=index, columns=columns, dtype=dtype)
+                if copy:
+                    pdf = pdf.copy()
+            else:
+                pdf = pd.DataFrame(data, index=index, columns=columns, dtype=dtype, copy=copy)
             if num_partitions is not None:
                 chunk_size = ceildiv(len(pdf), num_partitions)
             df = from_pandas_df(pdf, chunk_size=chunk_size, gpu=gpu, sparse=sparse)
@@ -93,9 +99,6 @@ class DataFrame(_Frame, metaclass=InitializerMeta):
 class Series(_Series, metaclass=InitializerMeta):
     def __init__(self, data=None, index=None, dtype=None, name=None, copy=False,
                  chunk_size=None, gpu=None, sparse=None, num_partitions=None):
-        # make sure __getattr__ does not result in stack overflow
-        self._data = None
-
         if dtype is not None:
             dtype = np.dtype(dtype)
 
@@ -120,7 +123,12 @@ class Series(_Series, metaclass=InitializerMeta):
                 series = data
             need_repart = num_partitions is not None
         else:
-            pd_series = pd.Series(data, index=index, dtype=dtype, name=name, copy=copy)
+            if is_cudf(data) or is_cupy(data):  # pragma: no cover
+                pd_series = cudf.Series(data, index=index, dtype=dtype, name=name)
+                if copy:
+                    pd_series = pd_series.copy()
+            else:
+                pd_series = pd.Series(data, index=index, dtype=dtype, name=name, copy=copy)
             if num_partitions is not None:
                 chunk_size = ceildiv(len(pd_series), num_partitions)
             series = from_pandas_series(pd_series, chunk_size=chunk_size, gpu=gpu, sparse=sparse)
@@ -138,9 +146,6 @@ class Index(_Index, metaclass=InitializerMeta):
     def __init__(self, data=None, dtype=None, copy=False, name=None, tupleize_cols=True,
                  chunk_size=None, gpu=None, sparse=None, names=None, num_partitions=None,
                  store_data=False):
-        # make sure __getattr__ does not result in stack overflow
-        self._data = None
-
         need_repart = False
         if isinstance(data, INDEX_TYPE):
             if not hasattr(data, 'data'):
@@ -150,15 +155,19 @@ class Index(_Index, metaclass=InitializerMeta):
                 index = data
             need_repart = num_partitions is not None
         else:
-            if isinstance(data, (Base, Entity)):
+            if isinstance(data, ENTITY_TYPE):
                 name = name if name is not None else getattr(data, 'name', None)
                 index = from_tileable_index(data, dtype=dtype, name=name, names=names)
                 need_repart = num_partitions is not None
             else:
-                if not isinstance(data, pd.Index):
+                if not is_index(data):
                     name = name if name is not None else getattr(data, 'name', None)
-                    pd_index = pd.Index(data=data, dtype=dtype, copy=copy, name=name,
-                                        tupleize_cols=tupleize_cols)
+                    xdf = cudf if is_cudf(data) or is_cupy(data) else pd
+                    try:
+                        pd_index = xdf.Index(data=data, dtype=dtype, copy=copy, name=name,
+                                             tupleize_cols=tupleize_cols)
+                    except TypeError:  # pragma: no cover
+                        pd_index = xdf.Index(data=data, dtype=dtype, copy=copy, name=name)
                 else:
                     pd_index = data
 
@@ -170,35 +179,3 @@ class Index(_Index, metaclass=InitializerMeta):
         if need_repart:
             index = index.rebalance(num_partitions=num_partitions)
         super().__init__(index.data)
-
-
-def named_dataframe(name, session=None):
-    from ..session import Session
-    from .utils import parse_index
-
-    sess = session or Session.default_or_local()
-    tileable_infos = sess.get_named_tileable_infos(name=name)
-    fetch_op = DataFrameFetch(output_types=[OutputType.dataframe])
-    extra_meta = tileable_infos.tileable_extra_meta
-    for param, store in zip(['index_value', 'columns_value'], ['False', 'True']):
-        if param in extra_meta:
-            extra_meta[param] = parse_index(extra_meta[param],
-                                            store_data=store)
-    return fetch_op.new_dataframe([], shape=tileable_infos.tileable_shape,
-                                  _key=tileable_infos.tileable_key,
-                                  kws=[extra_meta])
-
-
-def named_series(name, session=None):
-    from ..session import Session
-    from .utils import parse_index
-
-    sess = session or Session.default_or_local()
-    tileable_infos = sess.get_named_tileable_infos(name=name)
-    fetch_op = DataFrameFetch(output_types=[OutputType.series])
-    extra_meta = tileable_infos.tileable_extra_meta
-    if 'index_value' in extra_meta:
-        extra_meta['index_value'] = parse_index(extra_meta['index_value'])
-    return fetch_op.new_series([], shape=tileable_infos.tileable_shape,
-                               _key=tileable_infos.tileable_key,
-                               kws=[extra_meta])
