@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,19 +27,21 @@ class DataFrameAny(DataFrameReductionOperand, DataFrameReductionMixin):
         return True
 
 
-def any_series(series, axis=None, bool_only=None, skipna=None, level=None, combine_size=None):
+def any_series(series, axis=None, bool_only=None, skipna=None, level=None,
+               combine_size=None, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameAny(axis=axis, skipna=skipna, level=level, bool_only=bool_only,
                       combine_size=combine_size, output_types=[OutputType.scalar],
-                      use_inf_as_na=use_inf_as_na)
+                      use_inf_as_na=use_inf_as_na, method=method)
     return op(series)
 
 
-def any_dataframe(df, axis=None, bool_only=None, skipna=None, level=None, combine_size=None):
+def any_dataframe(df, axis=None, bool_only=None, skipna=None, level=None,
+                  combine_size=None, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameAny(axis=axis, skipna=skipna, level=level, bool_only=bool_only,
                       combine_size=combine_size, output_types=[OutputType.series],
-                      use_inf_as_na=use_inf_as_na)
+                      use_inf_as_na=use_inf_as_na, method=method)
     return op(df)
 
 
