@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,10 +19,12 @@ from typing import Any, Callable, Dict, List, NamedTuple, Optional
 import numpy as np
 import pandas as pd
 
-from ...core import OutputType, Entity, Base
-from ...operands import OperandStage
-from ...utils import tokenize, is_build_mode, is_kernel_mode, enter_mode, recursive_tile
-from ...serialize import BoolField, AnyField, DataTypeField, Int32Field
+from ...core import OutputType, ENTITY_TYPE, is_build_mode, \
+    is_kernel_mode, enter_mode, recursive_tile
+from ...core.operand import OperandStage
+from ...utils import tokenize
+from ...serialization.serializables import BoolField, AnyField, \
+    DataTypeField, Int32Field, StringField
 from ..core import SERIES_TYPE
 from ..utils import parse_index, build_df, build_empty_df, build_series, \
     build_empty_series, validate_axis
@@ -37,17 +39,19 @@ class DataFrameReductionOperand(DataFrameOperand):
     _bool_only = BoolField('bool_only')
     _min_count = Int32Field('min_count')
     _use_inf_as_na = BoolField('use_inf_as_na')
+    _method = StringField('method')
 
     _dtype = DataTypeField('dtype')
     _combine_size = Int32Field('combine_size')
 
     def __init__(self, axis=None, skipna=None, level=None, numeric_only=None, bool_only=None,
-                 min_count=None, stage=None, dtype=None, combine_size=None, gpu=None,
-                 sparse=None, output_types=None, use_inf_as_na=None, **kw):
+                 min_count=None, dtype=None, combine_size=None, gpu=None,
+                 sparse=None, output_types=None, use_inf_as_na=None, method=None, **kw):
         super().__init__(_axis=axis, _skipna=skipna, _level=level, _numeric_only=numeric_only,
-                         _bool_only=bool_only, _min_count=min_count, _stage=stage, _dtype=dtype,
+                         _bool_only=bool_only, _min_count=min_count, _dtype=dtype,
                          _combine_size=combine_size, _gpu=gpu, _sparse=sparse,
-                         _output_types=output_types, _use_inf_as_na=use_inf_as_na, **kw)
+                         _output_types=output_types, _use_inf_as_na=use_inf_as_na,
+                         _method=method, **kw)
 
     @property
     def axis(self):
@@ -89,6 +93,10 @@ class DataFrameReductionOperand(DataFrameOperand):
     def is_atomic(self):
         return False
 
+    @property
+    def method(self):
+        return self._method
+
     def get_reduction_args(self, axis=None):
         args = dict(skipna=self.skipna)
         if self.inputs[0].ndim > 1:
@@ -108,9 +116,9 @@ class DataFrameCumReductionOperand(DataFrameOperand):
     _dtype = DataTypeField('dtype')
 
     def __init__(self, axis=None, skipna=None, dtype=None, gpu=None, sparse=None,
-                 output_types=None, use_inf_as_na=None, stage=None, **kw):
+                 output_types=None, use_inf_as_na=None, **kw):
         super().__init__(_axis=axis, _skipna=skipna, _dtype=dtype, _gpu=gpu, _sparse=sparse,
-                         _output_types=output_types, _stage=stage, _use_inf_as_na=use_inf_as_na, **kw)
+                         _output_types=output_types, _use_inf_as_na=use_inf_as_na, **kw)
 
     @property
     def axis(self):
@@ -165,7 +173,7 @@ class DataFrameReductionMixin(DataFrameOperandMixin):
             output_type = OutputType.scalar
             dtypes, index = out_df.dtype, None
 
-        out_df = recursive_tile(in_df.agg(
+        out_df = yield from recursive_tile(in_df.agg(
             cls.get_reduction_callable(op), axis=op.axis or 0, _numeric_only=op.numeric_only,
             _bool_only=op.bool_only, _combine_size=op.combine_size, _output_type=output_type,
             _dtypes=dtypes, _index=index
@@ -173,7 +181,8 @@ class DataFrameReductionMixin(DataFrameOperandMixin):
         return [out_df]
 
     def _call_groupby_level(self, df, level):
-        return df.groupby(level=level).agg(self.get_reduction_callable(self))
+        return df.groupby(level=level).agg(self.get_reduction_callable(self),
+                                           method=self.method)
 
     def _call_dataframe(self, df):
         axis = getattr(self, 'axis', None) or 0
@@ -282,7 +291,7 @@ class DataFrameCumReductionMixin(DataFrameOperandMixin):
             to_concat_chunks.append(summary_chunks[j])
 
         new_chunk_op = op.copy().reset_key()
-        new_chunk_op._stage = OperandStage.combine
+        new_chunk_op.stage = OperandStage.combine
         return new_chunk_op.new_chunk(to_concat_chunks, **c.params)
 
     @classmethod
@@ -297,7 +306,7 @@ class DataFrameCumReductionMixin(DataFrameOperandMixin):
         summary_chunks = np.empty(in_df.chunk_shape, dtype=np.object)
         for c in in_df.chunks:
             new_chunk_op = op.copy().reset_key()
-            new_chunk_op._stage = OperandStage.map
+            new_chunk_op.stage = OperandStage.map
             if op.axis == 1:
                 summary_shape = (c.shape[0], 1)
             else:
@@ -335,7 +344,7 @@ class DataFrameCumReductionMixin(DataFrameOperandMixin):
         summary_chunks = np.empty(in_series.chunk_shape, dtype=np.object)
         for c in in_series.chunks:
             new_chunk_op = op.copy().reset_key()
-            new_chunk_op._stage = OperandStage.map
+            new_chunk_op.stage = OperandStage.map
             summary_chunks[c.index] = new_chunk_op.new_chunk([c], shape=(1,), dtype=series.dtype)
 
         # combine summaries into results
@@ -525,7 +534,7 @@ class ReductionSteps(NamedTuple):
     post_funcs: List[ReductionPostStep]
 
 
-# lookup table for numpy arithmetics in pandas
+# lookup table for numpy arithmetic operands in pandas
 _func_name_converts = dict(
     greater='gt',
     greater_equal='ge',
@@ -590,7 +599,7 @@ class ReductionCompiler:
                               n, cell in zip(func_code.co_freevars, func.__closure__)})
         # external Mars objects shall not be referenced
         for var_name, val in func_vars.items():
-            if isinstance(val, (Base, Entity)):
+            if isinstance(val, ENTITY_TYPE):
                 raise ValueError(f'Variable {var_name} used by {func.__name__} '
                                  'cannot be a Mars object')
 
@@ -680,7 +689,7 @@ class ReductionCompiler:
             func_ret = self._build_mock_return_object(func, object, ndim=1)
         output_limit = getattr(func, 'output_limit', None) or 1
 
-        if not isinstance(func_ret, (Base, Entity)):
+        if not isinstance(func_ret, ENTITY_TYPE):
             raise ValueError(f'Custom function should return a Mars object, not {type(func_ret)}')
         if func_ret.ndim >= ndim:
             raise ValueError('Function not a reduction')
