@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@ import numpy as np
 from ... import opcodes
 from ...config import options
 from ...core import OutputType
-from ...serialize import BoolField
+from ...serialization.serializables import BoolField
 from .core import DataFrameReductionOperand, DataFrameReductionMixin
 
 
@@ -63,18 +63,20 @@ class DataFrameKurtosis(DataFrameReductionOperand, DataFrameReductionMixin):
         return kurt
 
 
-def kurt_series(df, axis=None, skipna=None, level=None, combine_size=None, bias=False, fisher=True):
+def kurt_series(df, axis=None, skipna=None, level=None, combine_size=None, bias=False,
+                fisher=True, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameKurtosis(axis=axis, skipna=skipna, level=level, combine_size=combine_size,
                            bias=bias, fisher=fisher, output_types=[OutputType.scalar],
-                           use_inf_as_na=use_inf_as_na)
+                           use_inf_as_na=use_inf_as_na, method=method)
     return op(df)
 
 
 def kurt_dataframe(df, axis=None, skipna=None, level=None, numeric_only=None, combine_size=None,
-                   bias=False, fisher=True):
+                   bias=False, fisher=True, method=None):
     use_inf_as_na = options.dataframe.mode.use_inf_as_na
     op = DataFrameKurtosis(axis=axis, skipna=skipna, level=level, numeric_only=numeric_only,
                            bias=bias, fisher=fisher, combine_size=combine_size,
-                           output_types=[OutputType.series], use_inf_as_na=use_inf_as_na)
+                           output_types=[OutputType.series], use_inf_as_na=use_inf_as_na,
+                           method=method)
     return op(df)
