@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,6 @@ import numpy as np
 
 from ... import opcodes as OperandDef
 from ...core import OutputType
-from ...serialize import StringField
 from ...tensor.core import TensorOrder
 from ...utils import lazy_import
 from .core import DataFrameReductionOperand, DataFrameReductionMixin, CustomReduction
@@ -40,15 +39,6 @@ class DataFrameUnique(DataFrameReductionOperand, DataFrameReductionMixin):
     _op_type_ = OperandDef.UNIQUE
     _func_name = 'unique'
 
-    _method = StringField('method')
-
-    def __init__(self, method=None, **kw):
-        super().__init__(_method=method, **kw)
-
-    @property
-    def method(self):
-        return self._method
-
     @classmethod
     def get_reduction_callable(cls, op):
         return UniqueReduction(name=cls._func_name, is_gpu=op.is_gpu())
@@ -56,7 +46,7 @@ class DataFrameUnique(DataFrameReductionOperand, DataFrameReductionMixin):
     @classmethod
     def tile(cls, op):
         if op.method == 'tree':
-            return super().tile(op)
+            return (yield from super().tile(op))
         else:
             raise NotImplementedError(f"Method {op.method} hasn't been supported")
 
