@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 import pandas as pd
 
 from ... import opcodes as OperandDef
-from ...core import OutputType
-from ...serialize import ListField, BoolField
+from ...core import OutputType, recursive_tile
+from ...serialization.serializables import ListField, BoolField
 from ...tensor.base.sort import _validate_sort_psrs_kinds
 from ..utils import parse_index, validate_axis, build_concatenated_rows_frame, standardize_range_index
 from ..operands import DATAFRAME_TYPE
@@ -42,7 +42,7 @@ class DataFrameSortIndex(DataFrameSortOperand, DataFramePSRSOperandMixin):
         return self._sort_remaining
 
     @classmethod
-    def tile(cls, op):
+    def _tile(cls, op):
         df = op.inputs[0]
 
         if op.axis == 0:
@@ -78,12 +78,12 @@ class DataFrameSortIndex(DataFrameSortOperand, DataFramePSRSOperandMixin):
                 if op.na_position != 'last':  # pragma: no cover
                     raise NotImplementedError('Only support puts NaNs at the end.')
                 # use parallel sorting by regular sampling
-                return cls._tile_psrs(op, df)
+                return (yield from cls._tile_psrs(op, df))
         else:
             assert op.axis == 1
 
             sorted_columns = list(df.columns_value.to_pandas().sort_values(ascending=op.ascending))
-            r = [df[sorted_columns]._inplace_tile()]
+            r = [(yield from recursive_tile(df[sorted_columns]))]
             if op.ignore_index:
                 out = op.outputs[0]
                 chunks = standardize_range_index(r[0].chunks, axis=0)
@@ -94,9 +94,12 @@ class DataFrameSortIndex(DataFrameSortOperand, DataFramePSRSOperandMixin):
             return r
 
     @classmethod
-    def execute(cls, ctx, op):
+    def execute(cls, ctx, op: "DataFrameSortIndex"):
         in_data = ctx[op.inputs[0].key]
-        ctx[op.outputs[0].key] = execute_sort_index(in_data, op)
+        result = execute_sort_index(in_data, op)
+        if op.nrows is not None:
+            result = result.head(op.nrows)
+        ctx[op.outputs[0].key] = result
 
     def _call_dataframe(self, df):
         if self.ignore_index:
