@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,16 +14,16 @@
 
 import math
 from collections import OrderedDict
-from distutils.version import LooseVersion
 
 import pandas as pd
 
-from ....serialize import Int64Field, BoolField, Int32Field, Float64Field
+from ....lib.version import parse as parse_version
+from ....serialization.serializables import Int64Field, BoolField, Int32Field, Float64Field
 from ...utils import validate_axis
 from ..core import Window
 
-_default_min_period_1 = LooseVersion(pd.__version__) >= '1.1.0'
-_pd_1_3_repr = LooseVersion(pd.__version__) >= '1.3.0'
+_default_min_period_1 = parse_version(pd.__version__) >= parse_version('1.1.0')
+_pd_1_3_repr = parse_version(pd.__version__) >= parse_version('1.3.0')
 
 
 class EWM(Window):
@@ -88,7 +88,7 @@ class EWM(Window):
 
         params = self.params
         params['alpha_ignore_na'] = params.pop('ignore_na', False)
-        params['validate_columns'] = LooseVersion(pd.__version__) < '1.0.0'
+        params['validate_columns'] = False
         op = DataFrameEwmAgg(func=func, **params)
         return op(self)
 
