@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,12 +15,12 @@
 import numpy as np
 
 from ... import opcodes
-from ...operands import OutputType, OperandStage
-from ...serialize import KeyField, Int32Field, BoolField
+from ...core import OutputType, recursive_tile
+from ...core.operand import OperandStage
+from ...serialization.serializables import KeyField, Int32Field, BoolField
 from ...tensor.array_utils import as_same_device, device, cp, sparse
 from ...tensor.core import TensorOrder
-from ...tiles import TilesError
-from ...utils import check_chunks_unknown_shape
+from ...utils import has_unknown_shape
 from ..operands import LearnOperand, LearnOperandMixin
 from ._k_means_common import _execute_merge_update, _relocate_empty_clusters
 from ._k_means_elkan import init_bounds_dense, init_bounds_sparse, \
@@ -98,12 +98,13 @@ class KMeansElkanInitBounds(LearnOperand, LearnOperandMixin):
     @classmethod
     def tile(cls, op: "KMeansElkanInitBounds"):
         # unify chunks on axis 0
-        check_chunks_unknown_shape(
-            [op.centers, op.center_half_distances], TilesError)
+        if has_unknown_shape(op.centers, op.center_half_distances):
+            yield
         x = op.x
-        centers = op.centers.rechunk(op.centers.shape)._inplace_tile()
-        center_half_distances = op.center_half_distances.rechunk(
-            op.center_half_distances.shape)._inplace_tile()
+        centers = yield from recursive_tile(
+            op.centers.rechunk(op.centers.shape))
+        center_half_distances = yield from recursive_tile(
+            op.center_half_distances.rechunk(op.center_half_distances.shape))
 
         out_chunks = [list() for _ in range(op.output_limit)]
         for c in x.chunks:
@@ -205,14 +206,13 @@ class KMeansElkanUpdate(LearnOperand, LearnOperandMixin):
     def __init__(self, x=None, sample_weight=None, centers_old=None,
                  center_half_distances=None, distance_next_center=None,
                  labels=None, upper_bounds=None, lower_bounds=None, update_centers=None,
-                 n_clusters=None, output_types=None, stage=None, **kw):
+                 n_clusters=None, output_types=None, **kw):
         super().__init__(_x=x, _sample_weight=sample_weight, _centers_old=centers_old,
                          _center_half_distances=center_half_distances,
                          _distance_next_center=distance_next_center,
                          _labels=labels, _upper_bounds=upper_bounds,
                          _lower_bounds=lower_bounds, _update_centers=update_centers,
-                         _n_clusters=n_clusters, _output_types=output_types,
-                         _stage=stage, **kw)
+                         _n_clusters=n_clusters, _output_types=output_types, **kw)
         if self._output_types is None:
             self._output_types = [OutputType.tensor] * self.output_limit
 
@@ -258,7 +258,7 @@ class KMeansElkanUpdate(LearnOperand, LearnOperandMixin):
 
     @property
     def output_limit(self):
-        return 5 if self._stage != OperandStage.reduce else 2
+        return 5 if self.stage != OperandStage.reduce else 2
 
     @property
     def _input_fields(self):
@@ -268,7 +268,7 @@ class KMeansElkanUpdate(LearnOperand, LearnOperandMixin):
 
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
-        if self._stage != OperandStage.reduce:
+        if self.stage != OperandStage.reduce:
             input_fields = self._input_fields
             assert len(input_fields) == len(self._inputs)
             inputs_iter = iter(inputs)
@@ -301,20 +301,22 @@ class KMeansElkanUpdate(LearnOperand, LearnOperandMixin):
 
     @classmethod
     def tile(cls, op: "KMeansElkanUpdate"):
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
         x = op.x
         if x.chunk_shape[1] != 1:  # pragma: no cover
-            x = x.rechunk({1: x.shape[1]})._inplace_tile()
-        sample_weight = op.sample_weight.rechunk({0: x.nsplits[0]})._inplace_tile()
-        labels = op.labels.rechunk({0: x.nsplits[0]})._inplace_tile()
-        upper_bounds = op.upper_bounds.rechunk({0: x.nsplits[0]})._inplace_tile()
-        lower_bounds = op.lower_bounds.rechunk({0: x.nsplits[0],
-                                                1: op.lower_bounds.shape[1]})._inplace_tile()
-        centers_old = op.centers_old.rechunk(op.centers_old.shape)._inplace_tile()
-        center_half_distances = op.center_half_distances.rechunk(
-            op.center_half_distances.shape)._inplace_tile()
-        distance_next_center = op.distance_next_center.rechunk(
-            op.distance_next_center.shape)._inplace_tile()
+            x = yield from recursive_tile(x.rechunk({1: x.shape[1]}))
+        sample_weight = yield from recursive_tile(op.sample_weight.rechunk({0: x.nsplits[0]}))
+        labels = yield from recursive_tile(op.labels.rechunk({0: x.nsplits[0]}))
+        upper_bounds = yield from recursive_tile(op.upper_bounds.rechunk({0: x.nsplits[0]}))
+        lower_bounds = yield from recursive_tile(
+            op.lower_bounds.rechunk({0: x.nsplits[0],
+                                     1: op.lower_bounds.shape[1]}))
+        centers_old = yield from recursive_tile(op.centers_old.rechunk(op.centers_old.shape))
+        center_half_distances = yield from recursive_tile(op.center_half_distances.rechunk(
+            op.center_half_distances.shape))
+        distance_next_center = yield from recursive_tile(op.distance_next_center.rechunk(
+            op.distance_next_center.shape))
 
         out_chunks = [list() for _ in range(op.output_limit)]
         for i in range(x.chunk_shape[0]):
@@ -324,7 +326,7 @@ class KMeansElkanUpdate(LearnOperand, LearnOperandMixin):
             upper_bounds_chunk = upper_bounds.cix[i, ]
             lower_bounds_chunk = lower_bounds.cix[i, 0]
             chunk_op = op.copy().reset_key()
-            chunk_op._stage = OperandStage.map
+            chunk_op.stage = OperandStage.map
             chunk_kws = list((
                 # labels
                 labels_chunk.params,
@@ -451,13 +453,12 @@ class KMeansElkanPostprocess(LearnOperand, LearnOperandMixin):
 
     def __init__(self, centers_old=None, centers_new=None,
                  center_shift=None, lower_bounds=None, upper_bounds=None,
-                 labels=None, weight_in_clusters=None, output_types=None,
-                 stage=None, **kw):
+                 labels=None, weight_in_clusters=None, output_types=None, **kw):
         super().__init__(_centers_old=centers_old, _centers_new=centers_new,
                          _center_shift=center_shift, _lower_bounds=lower_bounds,
                          _upper_bounds=upper_bounds, _labels=labels,
-                         _weight_in_clusters=weight_in_clusters, _output_types=output_types,
-                         _stage=stage, **kw)
+                         _weight_in_clusters=weight_in_clusters,
+                         _output_types=output_types, **kw)
         if self._output_types is None:
             self._output_types = [OutputType.tensor] * self.output_limit
 
@@ -491,13 +492,13 @@ class KMeansElkanPostprocess(LearnOperand, LearnOperandMixin):
 
     @property
     def output_limit(self):
-        if self._stage is None:
+        if self.stage is None:
             # for tileable
             return 4
-        elif self._stage == OperandStage.combine:
+        elif self.stage == OperandStage.combine:
             return 2
         else:
-            assert self._stage == OperandStage.reduce
+            assert self.stage == OperandStage.reduce
             return 2
 
     @property
