@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,8 +13,9 @@
 # limitations under the License.
 
 from .run_script import run_pytorch_script
-from .dataset import MarsDataset
-from .sampler import MarsDistributedSampler, MarsRandomSampler
+from .dataset import MarsDataset  # noqa: F401 # pylint: disable=unused-import
+from .sampler import SequentialSampler, RandomSampler, \
+    SubsetRandomSampler, DistributedSampler  # noqa: F401 # pylint: disable=unused-import
 
 
 def register_op():
