@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,10 +16,11 @@ import numpy as np
 
 from .... import opcodes as OperandDef
 from .... import tensor as mt
-from ....serialize import KeyField, BoolField
+from ....config import options
+from ....core import recursive_tile
+from ....serialization.serializables import KeyField, BoolField
 from ....tensor.core import TensorOrder
-from ....tiles import TilesError
-from ....utils import recursive_tile, check_chunks_unknown_shape
+from ....utils import has_unknown_shape
 from ...utils import check_array
 from ...utils.extmath import row_norms
 from .core import PairwiseDistances
@@ -35,10 +36,9 @@ class EuclideanDistances(PairwiseDistances):
     _squared = BoolField('squared')
 
     def __init__(self, x=None, y=None, x_norm_squared=None, y_norm_squared=None,
-                 squared=None, dtype=None, gpu=None, **kw):
+                 squared=None, **kw):
         super().__init__(_x=x, _y=y, _x_norm_squared=x_norm_squared,
-                         _y_norm_squared=y_norm_squared, _squared=squared,
-                         _dtype=dtype, _gpu=gpu, **kw)
+                         _y_norm_squared=y_norm_squared, _squared=squared, **kw)
 
     @property
     def x(self):
@@ -115,13 +115,16 @@ class EuclideanDistances(PairwiseDistances):
         out = op.outputs[0]
 
         if X.dtype == np.float32:
-            check_chunks_unknown_shape([X, Y], TilesError)
+            if has_unknown_shape(X, Y):
+                yield
             # rechunk
             new_nsplit = max(max(X.nsplits[0]) // 2, 1)
-            X = recursive_tile(X.rechunk({0: new_nsplit}).astype(np.float64))
+            X = yield from recursive_tile(
+                X.rechunk({0: new_nsplit}).astype(np.float64))
             if Y is not X:
                 new_nsplit = max(max(Y.nsplits[0]) // 2, 1)
-                Y = recursive_tile(Y.rechunk({0: new_nsplit}).astype(np.float64))
+                Y = yield from recursive_tile(
+                    Y.rechunk({0: new_nsplit}).astype(np.float64))
 
         XX = op.x_norm_squared
         if XX is None:
@@ -130,7 +133,7 @@ class EuclideanDistances(PairwiseDistances):
         if YY is None:
             YY = row_norms(Y, squared=True)[np.newaxis, :]
 
-        X, Y = cls._adjust_chunk_sizes(X, Y, out)
+        X, Y = yield from cls._adjust_chunk_sizes(op, X, Y, out)
 
         distances = -2 * X.dot(Y.T)
         if distances.issparse():
@@ -144,7 +147,7 @@ class EuclideanDistances(PairwiseDistances):
 
         distances = distances if op.squared else mt.sqrt(distances)
         distances = distances.astype(out.dtype, copy=False)
-        return [recursive_tile(distances)]
+        return [(yield from recursive_tile(distances))]
 
 
 def euclidean_distances(X, Y=None, Y_norm_squared=None, squared=False,
@@ -227,6 +230,7 @@ def euclidean_distances(X, Y=None, Y_norm_squared=None, squared=False,
     X, Y = EuclideanDistances.check_pairwise_arrays(X, Y)
     op = EuclideanDistances(x=X, y=Y, x_norm_squared=X_norm_squared,
                             y_norm_squared=Y_norm_squared, squared=squared,
-                            dtype=np.dtype(dtype))
+                            dtype=np.dtype(dtype),
+                            chunk_store_limit=options.chunk_store_limit)
     return op(X, Y=Y, Y_norm_squared=Y_norm_squared,
               X_norm_squared=X_norm_squared)
