@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,13 +23,13 @@ import numpy as np
 
 from .... import opcodes
 from .... import tensor as mt
-from ....context import get_context, RunningMode
-from ....lib.filesystem import get_fs, LocalFileSystem
-from ....operands import OutputType, OperandStage
-from ....serialize import StringField, Int32Field, Int64Field, \
+from ....lib.filesystem import get_fs
+from ....core import OutputType
+from ....core.context import get_context
+from ....core.operand import OperandStage
+from ....serialization.serializables import StringField, Int32Field, Int64Field, \
     DictField, BytesField, TupleField, DataTypeField
-from ....tiles import TilesError
-from ....utils import check_chunks_unknown_shape, Timer
+from ....utils import has_unknown_shape, Timer
 from ...operands import LearnOperand, LearnOperandMixin
 from ..core import proxima, get_proxima_type, validate_tensor, \
     available_numpy_dtypes, rechunk_tensor, build_mmap_chunks
@@ -65,14 +65,14 @@ class ProximaBuilder(LearnOperand, LearnOperandMixin):
                  index_builder=None, index_builder_params=None,
                  index_converter=None, index_converter_params=None,
                  array_shape=None, array_dtype=None, offset=None,
-                 topk=None, storage_options=None, output_types=None, stage=None, **kw):
+                 topk=None, storage_options=None, output_types=None, **kw):
         super().__init__(_distance_metric=distance_metric, _index_path=index_path,
                          _dimension=dimension, _column_number=column_number, _index_builder=index_builder,
                          _index_builder_params=index_builder_params,
                          _array_shape=array_shape, _array_dtype=array_dtype, _offset=offset,
                          _index_converter=index_converter, _index_converter_params=index_converter_params,
                          _topk=topk, _storage_options=storage_options,
-                         _output_types=output_types, _stage=stage, **kw)
+                         _output_types=output_types, **kw)
         if self._output_types is None:
             self._output_types = [OutputType.object]
 
@@ -166,13 +166,6 @@ class ProximaBuilder(LearnOperand, LearnOperandMixin):
         if index_path is not None:
             fs = get_fs(index_path, op.storage_options)
 
-        # check index_path for distributed
-        if getattr(ctx, 'running_mode', None) == RunningMode.distributed:
-            if index_path is not None:
-                if isinstance(fs, LocalFileSystem):
-                    raise ValueError('`index_path` cannot be local file dir '
-                                     'for distributed index building')
-
         if index_path is not None:
             # check if the index path is empty
             try:
@@ -185,7 +178,8 @@ class ProximaBuilder(LearnOperand, LearnOperandMixin):
                 fs.mkdir(index_path)
 
         # make sure all inputs have known chunk sizes
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         if op.column_number:
             index_chunk_size = op.inputs[0].shape[0] // op.column_number
@@ -215,8 +209,8 @@ class ProximaBuilder(LearnOperand, LearnOperandMixin):
         final_out_chunks = []
         for j, chunks in enumerate(out_chunks):
             chunk_op = op.copy().reset_key()
-            chunk_op._stage = OperandStage.map
-            chunk_op._expect_worker = chunks[0].op.expect_worker
+            chunk_op.stage = OperandStage.map
+            chunk_op.expect_worker = chunks[0].op.expect_worker
             chunk_op._array_shape = chunks[0].op.total_shape
             chunk_op._array_dtype = chunks[0].dtype
             chunk_op._offset = offsets[j]
