@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from ...utils import is_build_mode
+from ...core import is_build_mode
 from .add import add, tree_add, TensorAdd, TensorTreeAdd
 from .subtract import subtract, TensorSubtract
 from .multiply import multiply, tree_multiply, TensorMultiply, TensorTreeMultiply
@@ -221,4 +221,3 @@ UNARY_UFUNC = {square, arcsinh, rint, sign, conj, tan, absolute, deg2rad, log, f
                cos, reciprocal, tanh, log1p, exp, arccos, arccosh, around, logical_not, conjugate,
                isfinite, isinf, isnan, signbit, spacing, floor, ceil, trunc, degrees, radians, angle,
                isreal, iscomplex, real, imag, fix, i0, sinc, nan_to_num}
-
