@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,12 +18,12 @@ import numpy as np
 
 from ... import opcodes as OperandDef
 from ...config import options
-from ...operands import OperandStage
+from ...core import recursive_tile
+from ...core.operand import OperandStage
 from ...lib import sparse
 from ...lib.sparse.core import get_array_module as get_sparse_array_module
-from ...serialize import BoolField, Int32Field, Int64Field
-from ...tiles import TilesError
-from ...utils import get_shuffle_input_keys_idxes, check_chunks_unknown_shape
+from ...serialization.serializables import BoolField, Int32Field, Int64Field
+from ...utils import has_unknown_shape
 from ..operands import TensorMapReduceOperand, TensorOperandMixin, TensorShuffleProxy
 from ..array_utils import as_same_device, device
 from ..core import TensorOrder
@@ -39,16 +39,13 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
     _axis = Int32Field('axis')
     _aggregate_size = Int32Field('aggregate_size')
 
-    _aggregate_id = Int32Field('aggregate_id')
     _start_pos = Int64Field('start_pos')
 
     def __init__(self, return_index=None, return_inverse=None, return_counts=None,
-                 axis=None, start_pos=None, stage=None, shuffle_key=None,
-                 dtype=None, gpu=None, aggregate_id=None, aggregate_size=None, **kw):
+                 axis=None, start_pos=None, aggregate_size=None, **kw):
         super().__init__(_return_index=return_index, _return_inverse=return_inverse,
                          _return_counts=return_counts, _axis=axis, _start_pos=start_pos,
-                         _aggregate_id=aggregate_id, _aggregate_size=aggregate_size,
-                         _stage=stage, _shuffle_key=shuffle_key, _dtype=dtype, _gpu=gpu, **kw)
+                         _aggregate_size=aggregate_size, **kw)
 
     @property
     def output_limit(self):
@@ -77,10 +74,6 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
     def aggregate_size(self):
         return self._aggregate_size
 
-    @property
-    def aggregate_id(self):
-        return self._aggregate_id
-
     @property
     def start_pos(self):
         return self._start_pos
@@ -170,16 +163,21 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
     def _tile_via_shuffle(cls, op):
         # rechunk the axes except the axis to do unique into 1 chunk
         inp = op.inputs[0]
+        if has_unknown_shape(inp):
+            yield
+
         if inp.ndim > 1:
             new_chunk_size = dict()
             for axis in range(inp.ndim):
                 if axis == op.axis:
                     continue
                 if np.isnan(inp.shape[axis]):
-                    raise TilesError(f'input tensor has unknown shape on axis {axis}')
+                    yield
                 new_chunk_size[axis] = inp.shape[axis]
-            check_chunks_unknown_shape([inp], TilesError)
-            inp = inp.rechunk(new_chunk_size)._inplace_tile()
+            if has_unknown_shape(inp):
+                yield
+            inp = yield from recursive_tile(
+                inp.rechunk(new_chunk_size))
 
         aggregate_size = op.aggregate_size
         if aggregate_size is None:
@@ -209,8 +207,9 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
                                      return_index=op.return_index,
                                      return_inverse=op.return_inverse,
                                      return_counts=op.return_counts,
-                                     axis=op.axis, aggregate_id=i,
-                                     shuffle_key=str(i))
+                                     axis=op.axis,
+                                     reducer_index=(i,),
+                                     reducer_phase='agg')
             kws = cls._gen_kws(op, inp, chunk=True, chunk_index=i)
             chunks = reduce_op.new_chunks([shuffle_chunk], kws=kws,
                                           order=op.outputs[0].order)
@@ -224,8 +223,10 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
                 dtype=map_inverse_chunks[0].dtype).new_chunk(map_inverse_chunks, shape=())
             inverse_chunks = []
             for j, cs in enumerate(unique_on_chunk_sizes):
-                chunk_op = TensorUniqueInverseReduce(dtype=map_inverse_chunks[0].dtype,
-                                                     shuffle_key=str(j))
+                chunk_op = TensorUnique(stage=OperandStage.reduce,
+                                        dtype=map_inverse_chunks[0].dtype,
+                                        reducer_index=(j,),
+                                        reducer_phase='inverse')
                 inverse_chunk = chunk_op.new_chunk([inverse_shuffle_chunk], shape=(cs,),
                                                    index=(j,))
                 inverse_chunks.append(inverse_chunk)
@@ -251,14 +252,14 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
         return new_op.new_tensors(op.inputs, kws=kws)
 
     @classmethod
-    def tile(cls, op):
+    def tile(cls, op: "TensorUnique"):
         if len(op.inputs[0].chunks) == 1:
             return cls._tile_one_chunk(op)
         else:
-            return cls._tile_via_shuffle(op)
+            return (yield from cls._tile_via_shuffle(op))
 
     @classmethod
-    def _execute_map(cls, ctx, op):
+    def _execute_map(cls, ctx, op: "TensorUnique"):
         (ar,), device_id, xp = as_same_device(
             [ctx[c.key] for c in op.inputs], device=op.device, ret_extra=True)
         n_reducer = op.aggregate_size
@@ -307,15 +308,13 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
                 # counts
                 if counts_ar is not None:
                     res.append(counts_ar[cond])
-                ctx[(op.outputs[0].key, str(reducer))] = tuple(res)
+                ctx[op.outputs[0].key, (reducer,)] = tuple(res)
 
     @classmethod
-    def _execute_reduce(cls, ctx, op):
-        in_chunk = op.inputs[0]
-        input_keys, input_indexes = get_shuffle_input_keys_idxes(in_chunk)
+    def _execute_agg_reduce(cls, ctx, op: "TensorUnique"):
+        input_indexes, input_data = zip(*list(op.iter_mapper_data_with_index(ctx)))
 
-        inputs = list(zip(*(ctx[(input_key, str(op.aggregate_id))]
-                            for input_key in input_keys)))
+        inputs = list(zip(*input_data))
         flatten, device_id, xp = as_same_device(
             list(itertools.chain(*inputs)), device=op.device, ret_extra=True)
         n_ret = len(inputs[0])
@@ -371,7 +370,7 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
                             r[0] = inverse_array[0]
                             r[1] = p[inverse_array[1]]
                         # return unique length and
-                        ctx[(indices_out_key, str(input_indexes[i][op.axis]))] = \
+                        ctx[indices_out_key, (input_indexes[i][op.axis],)] = \
                             results[0].shape[op.axis], r
                 # calc counts
                 if op.return_counts:
@@ -388,11 +387,29 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
                 pass
 
     @classmethod
-    def execute(cls, ctx, op):
+    def _execute_inverse_reduce(cls, ctx, op: "TensorUnique"):
+        out = op.outputs[0]
+        inputs = list(op.iter_mapper_data(ctx))
+        unique_sizes = [inp[0] for inp in inputs]
+        cum_unique_sizes = np.cumsum([0] + unique_sizes)
+        invs, device_id, xp = as_same_device([inp[1] for inp in inputs],
+                                             device=op.device, ret_extra=True)
+        with device(device_id):
+            ret = xp.empty(out.shape, dtype=out.dtype)
+            for i, inv in enumerate(invs):
+                ret[inv[0]] = cum_unique_sizes[i] + inv[1]
+            ctx[out.key] = ret
+
+    @classmethod
+    def execute(cls, ctx, op: "TensorUnique"):
         if op.stage == OperandStage.map:
             cls._execute_map(ctx, op)
         elif op.stage == OperandStage.reduce:
-            cls._execute_reduce(ctx, op)
+            if op.reducer_phase == 'agg':
+                cls._execute_agg_reduce(ctx, op)
+            else:
+                assert op.reducer_phase == 'inverse'
+                cls._execute_inverse_reduce(ctx, op)
         else:
             (ar,), device_id, xp = as_same_device(
                 [ctx[c.key] for c in op.inputs], device=op.device, ret_extra=True)
@@ -415,29 +432,6 @@ class TensorUnique(TensorMapReduceOperand, TensorOperandMixin):
                     ctx[out.key] = result
 
 
-class TensorUniqueInverseReduce(TensorMapReduceOperand, TensorOperandMixin):
-    _op_type_ = OperandDef.UNIQUE_INVERSE_REDUCE
-
-    def __init__(self, shuffle_key=None, dtype=None, gpu=None, **kw):
-        super().__init__(_stage=OperandStage.reduce, _shuffle_key=shuffle_key,
-                         _dtype=dtype, _gpu=gpu, **kw)
-
-    @classmethod
-    def execute(cls, ctx, op):
-        out = op.outputs[0]
-        input_keys, _ = get_shuffle_input_keys_idxes(op.inputs[0])
-        inputs = [ctx[(inp_key, op.shuffle_key)] for inp_key in input_keys]
-        unique_sizes = [inp[0] for inp in inputs]
-        cum_unique_sizes = np.cumsum([0] + unique_sizes)
-        invs, device_id, xp = as_same_device([inp[1] for inp in inputs],
-                                             device=op.device, ret_extra=True)
-        with device(device_id):
-            ret = xp.empty(out.shape, dtype=out.dtype)
-            for i, inv in enumerate(invs):
-                ret[inv[0]] = cum_unique_sizes[i] + inv[1]
-            ctx[out.key] = ret
-
-
 def unique(ar, return_index=False, return_inverse=False, return_counts=False,
            axis=None, aggregate_size=None):
     """
