@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,9 +19,8 @@ import itertools
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import KeyField, StringField
-from ...utils import check_chunks_unknown_shape
-from ...tiles import TilesError
+from ...serialization.serializables import KeyField, StringField
+from ...utils import has_unknown_shape
 from ..core import Tensor, TensorOrder
 from ..utils import broadcast_shape, check_out_param, unify_chunks, check_order
 from ..array_utils import device, as_same_device, is_sparse_module
@@ -38,8 +37,8 @@ class TensorMatmul(TensorOperand, TensorOperandMixin):
     _casting = StringField('casting')
     _order = StringField('order')
 
-    def __init__(self, dtype=None, sparse=False, casting=None, order=None,  **kw):
-        super().__init__(_dtype=dtype, _sparse=sparse, _casting=casting, _order=order, **kw)
+    def __init__(self, casting=None, order=None,  **kw):
+        super().__init__(_casting=casting, _order=order, **kw)
         if self._casting is None:
             self._casting = 'same_kind'
         if self._order is None:
@@ -132,8 +131,9 @@ class TensorMatmul(TensorOperand, TensorOperandMixin):
         # the axes to align on
         a_axes = list(range(a.ndim - 2))[::-1] + [tensor.ndim - 2, tensor.ndim - 1]
         b_axes = list(range(b.ndim - 2))[::-1] + [tensor.ndim - 1, tensor.ndim]
-        check_chunks_unknown_shape(op.inputs, TilesError)
-        a, b = unify_chunks((a, a_axes), (b, b_axes))
+        if has_unknown_shape(a, b):
+            yield
+        a, b = yield from unify_chunks((a, a_axes), (b, b_axes))
 
         get_nsplit = lambda i: a.nsplits[i] if a.nsplits[i] != (1,) else b.nsplits[i]
         get_idx = lambda ch, idx: tuple(0 if ch.nsplits[j] == (1,) else ix for j, ix in enumerate(idx))
