@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,9 +20,8 @@ from collections.abc import Iterable
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import ValueType, KeyField, TupleField
-from ...utils import check_chunks_unknown_shape
-from ...tiles import TilesError
+from ...serialization.serializables import FieldTypes, KeyField, TupleField
+from ...utils import has_unknown_shape
 from ..utils import unify_chunks
 from ..array_utils import as_same_device, device, is_sparse_module
 from ..operands import TensorOperand, TensorOperandMixin
@@ -36,11 +35,11 @@ class TensorTensorDot(TensorOperand, TensorOperandMixin):
 
     _a = KeyField('a')
     _b = KeyField('b')
-    _a_axes = TupleField('a_axes', ValueType.int32)
-    _b_axes = TupleField('b_axes', ValueType.int32)
+    _a_axes = TupleField('a_axes', FieldTypes.int32)
+    _b_axes = TupleField('b_axes', FieldTypes.int32)
 
-    def __init__(self, a_axes=None, b_axes=None, dtype=None, sparse=False, **kw):
-        super().__init__(_a_axes=a_axes, _b_axes=b_axes, _dtype=dtype, _sparse=sparse, **kw)
+    def __init__(self, a_axes=None, b_axes=None, **kw):
+        super().__init__(_a_axes=a_axes, _b_axes=b_axes, **kw)
 
     @property
     def a(self):
@@ -91,8 +90,9 @@ class TensorTensorDot(TensorOperand, TensorOperandMixin):
         c = itertools.count(max(a.ndim, b.ndim))
         a_ax = tuple(a_axes.index(i) if i in a_axes else next(c) for i in range(a.ndim))
         b_ax = tuple(b_axes.index(i) if i in b_axes else next(c) for i in range(b.ndim))
-        check_chunks_unknown_shape(op.inputs, TilesError)
-        a, b = unify_chunks((a, a_ax), (b, b_ax))
+        if has_unknown_shape(*op.inputs):
+            yield
+        a, b = yield from unify_chunks((a, a_ax), (b, b_ax))
         out = op.outputs[0]
 
         a_output_indexes = [range(len(a.nsplits[i])) for i in range(a.ndim) if i not in a_axes]
