@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,22 +17,23 @@
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import AnyField
+from ...serialization.serializables import AnyField
+from ..utils import gen_random_seeds
 from .core import TensorRandomOperandMixin, handle_array, TensorDistribution
 
 
 class TensorRandGamma(TensorDistribution, TensorRandomOperandMixin):
-    __slots__ = '_shape', '_scale', '_size'
     _input_fields_ = ['_shape', '_scale']
     _op_type_ = OperandDef.RAND_GAMMA
 
+    _fields_ = '_shape', '_scale', '_size'
     _shape = AnyField('shape')
     _scale = AnyField('scale')
     _func_name = 'gamma'
 
-    def __init__(self, state=None, size=None, dtype=None, gpu=None, **kw):
+    def __init__(self, size=None, dtype=None, **kw):
         dtype = np.dtype(dtype) if dtype is not None else dtype
-        super().__init__(_state=state, _size=size, _dtype=dtype, _gpu=gpu, **kw)
+        super().__init__(_size=size, dtype=dtype, **kw)
 
     @property
     def shape(self):
@@ -129,5 +130,6 @@ def gamma(random_state, shape, scale=1.0, size=None, chunk_size=None, gpu=None,
         dtype = np.random.RandomState().gamma(
             handle_array(shape), handle_array(scale), size=(0,)).dtype
     size = random_state._handle_size(size)
-    op = TensorRandGamma(state=random_state.to_numpy(), size=size, gpu=gpu, dtype=dtype)
+    seed = gen_random_seeds(1, random_state.to_numpy())[0]
+    op = TensorRandGamma(seed=seed, size=size, gpu=gpu, dtype=dtype)
     return op(shape, scale, chunk_size=chunk_size)
