@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,9 +17,8 @@ import itertools
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import KeyField, AnyField, BoolField, Int32Field, Int64Field
-from ...utils import check_chunks_unknown_shape
-from ...tiles import TilesError
+from ...serialization.serializables import KeyField, AnyField, Int32Field, Int64Field
+from ...utils import has_unknown_shape
 from ..utils import calc_sliced_size
 from ..operands import TensorHasInput, TensorOperandMixin
 from ..datasource import tensor as astensor
@@ -33,14 +32,10 @@ class TensorRechunk(TensorHasInput, TensorOperandMixin):
     _chunk_size = AnyField('chunk_size')
     _threshold = Int32Field('threshold')
     _chunk_size_limit = Int64Field('chunk_size_limit')
-    _reassign_worker = BoolField('reassign_worker')
 
-    def __init__(self, chunk_size=None, threshold=None, chunk_size_limit=None,
-                 reassign_worker=None, dtype=None, sparse=False, **kw):
+    def __init__(self, chunk_size=None, threshold=None, chunk_size_limit=None, **kw):
         super().__init__(_chunk_size=chunk_size, _threshold=threshold,
-                         _chunk_size_limit=chunk_size_limit,
-                         _reassign_worker=reassign_worker,
-                         _dtype=dtype, _sparse=sparse, **kw)
+                         _chunk_size_limit=chunk_size_limit, **kw)
 
     @property
     def input(self):
@@ -67,7 +62,8 @@ class TensorRechunk(TensorHasInput, TensorOperandMixin):
 
     @classmethod
     def tile(cls, op):
-        check_chunks_unknown_shape(op.inputs, TilesError)
+        if has_unknown_shape(*op.inputs):
+            yield
 
         tensor = astensor(op.input)
         chunk_size = get_nsplits(tensor, op.chunk_size, tensor.dtype.itemsize)
@@ -84,7 +80,7 @@ class TensorRechunk(TensorHasInput, TensorOperandMixin):
 
         if op.reassign_worker:
             for c in tensor.chunks:
-                c.op._reassign_worker = True
+                c.op.reassign_worker = True
 
         return [tensor]
 
@@ -92,13 +88,7 @@ class TensorRechunk(TensorHasInput, TensorOperandMixin):
 def rechunk(tensor, chunk_size, threshold=None, chunk_size_limit=None,
             reassign_worker=False):
     if not any(np.isnan(s) for s in tensor.shape) and not tensor.is_coarse():
-        try:
-            check_chunks_unknown_shape([tensor], ValueError)
-        except ValueError:
-            # due to reason that tensor has unknown chunk shape,
-            # just ignore to hand over to operand
-            pass
-        else:
+        if not has_unknown_shape(tensor):
             # do client check only when tensor has no unknown shape,
             # otherwise, recalculate chunk_size in `tile`
             chunk_size = get_nsplits(tensor, chunk_size, tensor.dtype.itemsize)
@@ -126,7 +116,7 @@ def compute_rechunk(tensor, chunk_size):
             chunk_index, chunk_slice = zip(*index_slices)
             old_chunk = tensor.cix[chunk_index]
             merge_chunk_shape = tuple(calc_sliced_size(s, chunk_slice[0]) for s in old_chunk.shape)
-            merge_chunk_op = TensorSlice(chunk_slice, dtype=old_chunk.dtype, sparse=old_chunk.op.sparse)
+            merge_chunk_op = TensorSlice(list(chunk_slice), dtype=old_chunk.dtype, sparse=old_chunk.op.sparse)
             merge_chunk = merge_chunk_op.new_chunk([old_chunk], shape=merge_chunk_shape,
                                                    index=merge_idx, order=tensor.order)
             to_merge.append(merge_chunk)
