@@ -1,4 +1,4 @@
-# Copyright 1999-2020 Alibaba Group Holding Ltd.
+# Copyright 1999-2021 Alibaba Group Holding Ltd.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,11 +17,11 @@ from collections.abc import Iterable
 import numpy as np
 
 from ... import opcodes as OperandDef
-from ...serialize import KeyField, AnyField, StringField, BoolField
-from ...core import Base, Entity
-from ...utils import check_chunks_unknown_shape, recursive_tile
-from ...tiles import TilesError
-from ...context import get_context
+from ...core import ENTITY_TYPE, recursive_tile
+from ...core.context import get_context
+from ...serialization.serializables import KeyField, AnyField, \
+    StringField, BoolField
+from ...utils import has_unknown_shape
 from ..datasource import tensor as astensor
 from ..base import moveaxis, where
 from ..indexing import take
@@ -194,11 +194,11 @@ class TensorQuantile(TensorOperand, TensorOperandMixin):
     _keepdims = BoolField('keepdims')
 
     def __init__(self, q=None, axis=None, out=None, overwrite_input=None,
-                 interpolation=None, keepdims=None, dtype=None, gpu=None, **kw):
+                 interpolation=None, keepdims=None, **kw):
         self.q_error_msg = kw.pop('q_error_msg', q_error_msg)
         super().__init__(_q=q, _axis=axis, _interpolation=interpolation,
                          _out=out, _overwrite_input=overwrite_input,
-                         _keepdims=keepdims, _dtype=dtype, _gpu=gpu, **kw)
+                         _keepdims=keepdims, **kw)
 
     def _set_inputs(self, inputs):
         super()._set_inputs(inputs)
@@ -294,25 +294,27 @@ class TensorQuantile(TensorOperand, TensorOperandMixin):
 
     @classmethod
     def tile(cls, op):
-        check_chunks_unknown_shape(op.inputs, TilesError)
         if isinstance(op.q, TENSOR_TYPE):
+            # trigger execution of `q`
+            yield op.q.chunks
+
             ctx = get_context()
             # get q's data
             q_chunk_keys = [c.key for c in op.q.chunks]
-            metas = ctx.get_chunk_metas(q_chunk_keys)
-            if any(meta is None for meta in metas):
-                raise TilesError('q has to be executed if it\' a tensor')
-            q_data = ctx.get_chunk_results(q_chunk_keys)
+            q_data = ctx.get_chunks_result(q_chunk_keys)
             op._q = q = np.concatenate(q_data)
             if not _quantile_is_valid(q):
                 raise ValueError(op.q_error_msg)
         else:
+            if has_unknown_shape(*op.inputs):
+                yield
             q = np.asarray(op.q)
 
         if len(op.a.chunks) == 1 and (op.out is None or len(op.out.chunks) == 1):
             return cls._tile_one_chunk(op, q)
         else:
-            return [recursive_tile(cls._tile(op, q))]
+            tiled = yield from recursive_tile(cls._tile(op, q))
+            return [tiled]
 
     @classmethod
     def execute(cls, ctx, op):
@@ -341,7 +343,7 @@ def _quantile_unchecked(a, q, axis=None, out=None, overwrite_input=False,
         # to handle the datetime-like dtype
         a = a.astype('i8')
         need_view_back = True
-    if isinstance(q, (Base, Entity)):
+    if isinstance(q, ENTITY_TYPE):
         q = astensor(q)
         # do check in tile
         q_input = q
@@ -476,7 +478,7 @@ def quantile(a, q, axis=None, out=None, overwrite_input=False,
         raise TypeError('quantile() got an unexpected keyword '
                         f'argument \'{next(iter(kw))}\'')
 
-    if not isinstance(q, (Base, Entity)):
+    if not isinstance(q, ENTITY_TYPE):
         q = np.asanyarray(q)
         # do check instantly if q is not a tensor
         if not _quantile_is_valid(q):
